
## Need to install R package glmnet for LASSO section
## install.packages("glmnet")

## Need to install R package lavaan for Confirmatory Factor Analysis
## install.packages("lavaan")

################################
## (Exploratory) Factor Analysis
################################


## read in the data
persDat <- read.table("personality.csv", sep =",")

## Brief Look at the data
head(persDat)


## Perform a Factor Analysis fitting
## no need to standardize; factanal standardizes for us
FAfit1 <- factanal(persDat, factors = 5, rotation = "none")


## Look at Fit output
FAfit1


## Vizualize Loadings
load = FAfit1$loadings[,1:2]
plot(load, type="n") # set up plot 
text(load,labels=names(persDat),cex=.7) # add variable names

## Perform a second Factor analysis fitting using varimax rotation
FAfit2 <- factanal(persDat, factors = 5, rotation = "varimax")

## Visualize Loads of Second Fit
load = FAfit2$loadings[,1:2]
plot(load, type="n") # set up plot 
text(load,labels=names(persDat),cex=.7) # add variable names

#################################
## (Confirmatory) Factor Analysis
#################################

## lavaan is the package we use for CFA
require(lavaan)

## Present the apriori model
HS.model <- ' visual  =~ x1 + x2 + x3 
              textual =~ x4 + x5 + x6
              speed   =~ x7 + x8 + x9 '


## Use CFA to fit the model
CFAfit <- cfa(HS.model, data = HolzingerSwineford1939)

## Examine fittings
summary(CFAfit)


########
## LASSO
########



## glmnet is the most popular package for running lasso
require(glmnet) 

## read in the data
memDat <- read.table("memory.csv", sep =",")

## Brief Look at the data
head(memDat)

## glmnet takes data in a y and x format
## so split data set into our response y, and covariate matrix x
## where x must be a matrx, not a data frame
y <- memDat$Y
x <- as.matrix(memDat[,-1]) ## the '-1' is dropping the first column, which is the y values

## Perform a LASSO fitting
## no need to standardize; glmnet standardizes for us
LASSOpath <- glmnet(y = y, x = x)


## Visualize the lasso path
plot(LASSOpath)


## LASSO fit selected by 5 fold cross-validation
## default setting is uses squared predictionerror
LASSOfit <- cv.glmnet(y = y, x = x, nfolds = 5)

## Plot cross validation error over considered models
plot(LASSOfit)

## get coefficient values of model with minimum cross validaiton error
coef(LASSOfit, s = "lambda.min")
